module kk{
	//コンパイル通すために追加
	declare function escape(arg?:any):any;
	declare function unescape(arg?:any):any;

	//original code from document.cookie in MDN
	//https://developer.mozilla.org/ja/docs/DOM/document.cookie
	export class CookieUtil{

		static getItem(sKey:string):any {
			if (!sKey || !this.hasItem(sKey)) { return null; }
			return unescape(document.cookie.replace(new RegExp("(?:^|.*;\\s*)" + escape(sKey).replace(/[\-\.\+\*]/g, "\\$&") + "\\s*\\=\\s*((?:[^;](?!;))*[^;]?).*"), "$1"));
		}

		static setItem(sKey:string, sValue:string, vEnd?:any, sPath?:string, sDomain?:string, bSecure:boolean = false):void {
			if (!sKey || /^(?:expires|max\-age|path|domain|secure)$/i.test(sKey)) { return; }
			var sExpires = "";
			if (vEnd) {
				switch (vEnd.constructor) {
					case Number:
						sExpires = vEnd === Infinity ? "; expires=Tue, 19 Jan 2038 03:14:07 GMT" : "; max-age=" + vEnd;
						break;
					case String:
						sExpires = "; expires=" + vEnd;
						break;
					case Date:
						sExpires = "; expires=" + vEnd.toUTCString();
						break;
				}
			}
			document.cookie = escape(sKey) + "=" + escape(sValue) + sExpires + (sDomain ? "; domain=" + sDomain : "") + (sPath ? "; path=" + sPath : "") + (bSecure ? "; secure" : "");
		}

		static removeItem(sKey:string, sPath?:string):void {
			if (!sKey || !this.hasItem(sKey)) { return; }
			document.cookie = escape(sKey) + "=; expires=Thu, 01 Jan 1970 00:00:00 GMT" + (sPath ? "; path=" + sPath : "");
		}
		static hasItem(sKey:string):boolean {
			return (new RegExp("(?:^|;\\s*)" + escape(sKey).replace(/[\-\.\+\*]/g, "\\$&") + "\\s*\\=")).test(document.cookie);
		}

		static keys():string[] {
			var aKeys = document.cookie.replace(/((?:^|\s*;)[^\=]+)(?=;|$)|^\s*|\s*(?:\=[^;]*)?(?:\1|$)/g, "").split(/\s*(?:\=[^;]*)?;\s*/);
			for (var nIdx = 0; nIdx < aKeys.length; nIdx++) { aKeys[nIdx] = unescape(aKeys[nIdx]); }
			return aKeys;
		}

		static checkEnabled():boolean{
			var result:boolean = false;
			try{
				var propName:string = 'testEnabled';
				CookieUtil.setItem(propName, "test");
				result = !(CookieUtil.getItem(propName) === null);
				if(result){
					CookieUtil.removeItem(propName);
				}
			}catch(err){
				return false;
			}
			return result;
		}
	}

	//有効期限のDateをつくるためのユーティリティ
	export class FutureDate{
		static getNow():Date{
			return new Date();
		}

		static afterSeconds(seconds:number):Date{
			var d:Date = FutureDate.getNow();
			d.setTime(d.getTime() + seconds * 1000);
			return d;
		}

		static afterMinutes(minutes:number):Date{
			var d:Date = FutureDate.getNow();
			d.setTime(d.getTime() + minutes * 60 * 1000);
			return d;
		}

		static afterHours(hours:number):Date{
			var d:Date = FutureDate.getNow();
			d.setTime(d.getTime() + hours * 60 * 60 * 1000);
			return d;
		}

		static afterDays(days:number):Date{
			var d:Date = FutureDate.getNow();
			d.setTime(d.getTime() + days * 24 * 60 * 60 * 1000);
			return d;
		}
	}
}
